﻿using System.ComponentModel;
using UserStageExample.FlexiService;

namespace UserStageExample.ViewModel
{
    /// <summary>
    /// Возможные состояния документа
    /// </summary>
    internal enum DocumentState
    {
        /// <summary>
        /// Не выбрано
        /// </summary>
        Untagged,

        /// <summary>
        /// Документ подтверждён
        /// </summary>
        Accepted,

        /// <summary>
        /// Документ отклонён
        /// </summary>
        Rejected
    }

    /// <summary>
    /// Контроллер документа
    /// </summary>
    internal class DocumentViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Отображаемый документ
        /// </summary>
        private Document _document;
        /// <summary>
        /// Состояние документа
        /// </summary>
        private DocumentState _state = DocumentState.Untagged;
        /// <summary>
        /// Возникает при изменении свойств контроллера документа
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Идентификатор документа
        /// </summary>
        public int Id { get; private set; }
        /// <summary>
        /// Исходный документ
        /// </summary>
        public Document Document
        {
            get { return _document; }
        }
        /// <summary>
        /// Задача, к которой принадлежит документ
        /// </summary>
        public TaskViewModel Task { get; set; }
        /// <summary>
        /// Название документа
        /// </summary>
        public string Name
        {
            get { return _document == null ? null : _document.PersonName(); }
        }
        /// <summary>
        /// Определяет, был ли документ принят
        /// </summary>
        public bool IsAccepted { get { return _state == DocumentState.Accepted; } }
        /// <summary>
        /// Определяет, был ли документ отклонён
        /// </summary>
        public bool IsRejected { get { return _state == DocumentState.Rejected; } }

        /// <summary>
        /// Состояние документа
        /// </summary>
        public DocumentState State
        {
            get { return _state; }
            set
            {
                _state = value;
                OnPropertyChanged("State");
                OnPropertyChanged("IsAccepted");
                OnPropertyChanged("IsRejected");
            }
        }

        /// <summary>
        /// Создаёт новый документ на основании модели
        /// </summary>
        /// <param name="id">Идентификатор документа</param>
        /// <param name="document">Модель документа</param>
        public DocumentViewModel(int id, Document document)
        {
            Id = id;
            _document = document;
        }

        /// <summary>
        /// Обновляет значения контроллера документа значениями модели
        /// </summary>
        /// <param name="document">Модель документа</param>
        public void Update(Document document)
        {
            var oldName = Name;
            _document = document;
            if (oldName != Name) OnPropertyChanged("Name");
        }

        /// <summary>
        /// Подтверждает документ, или отменяет подверждение
        /// </summary>
        public void Accept()
        {
            State = State == DocumentState.Accepted ? DocumentState.Untagged : DocumentState.Accepted;
        }

        /// <summary>
        /// Отклоняет документ или подтверждает отклонение
        /// </summary>
        public void Reject()
        {
            State = State == DocumentState.Rejected ? DocumentState.Untagged : DocumentState.Rejected;
        }

        /// <summary>
        /// Формирует сообщение PropertyChanged
        /// </summary>
        /// <param name="propertyName">Название свойства</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            var h = PropertyChanged;
            if (h != null) h.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}