﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using UserStageExample.FlexiService;

namespace UserStageExample.ViewModel
{
    /// <summary>
    /// Представляет задачу FlexiCapture
    /// </summary>
    internal class TaskViewModel
    {
        /// <summary>
        /// Идентификатор задачи в системе FlexiCapture
        /// </summary>
        public int Id { get; private set; }
        /// <summary>
        /// Идентификтор пакета FlexiCapture, из документов которого состоит задача
        /// </summary>
        public int BatchId { get; private set; }
        /// <summary>
        /// Набор контроллеров документов.
        /// </summary>
        public ObservableCollection<DocumentViewModel> Documents { get; private set; }
        /// <summary>
        /// Определяет, содержит ли задача неразобранные документы
        /// </summary>
        public bool HasUntagged { get { return Documents.Any(x => x.State == DocumentState.Untagged); } }

        /// <summary>
        /// Создаёт новый контроллер задачи на основании модели задачи
        /// </summary>
        /// <param name="id">Идентификатор задачи</param>
        /// <param name="task">Модель задачи</param>
        public TaskViewModel(int id, Task task)
        {
            Id = id;
            BatchId = task.BatchId;
            Documents = new ObservableCollection<DocumentViewModel>();
            UpdateDocuments(task.Documents);
        }

        /// <summary>
        /// Обновляет контроллер задачи значениями переданной модели
        /// </summary>
        /// <param name="task">Модель задачи</param>
        public void Update(Task task)
        {
            UpdateDocuments(task.Documents);
        }

        /// <summary>
        /// Обновляет список документов задачи на основании переданного
        /// </summary>
        /// <remarks>
        /// Метод сопоставляет отображаемые id документов с передаными id
        /// Если документ уже отображается - он обновляется, если нет - добавляется.
        /// Если отображаемый документ не передан - удаляется.
        /// </remarks>
        /// <param name="documents">Требуемый список документов</param>
        private void UpdateDocuments(IDictionary<int, Document> documents)
        {
            // Обновление существующих документов и удаление исчезнувших
            for (var i = 0; i < Documents.Count; i++)
            {
                var documentViewModel = Documents[i];
                Document document;
                if (documents.TryGetValue(documentViewModel.Id, out document))
                {
                    documents.Remove(documentViewModel.Id);
                    documentViewModel.Update(document);
                }
                else
                {
                    Documents.RemoveAt(i--);
                }
            }
            // Добавление новых документов
            foreach (var document in documents)
            {
                Add(new DocumentViewModel(document.Key, document.Value));
            }
        }

        /// <summary>
        /// Добавляет новый документ
        /// </summary>
        /// <param name="documentViewModel"></param>
        private void Add(DocumentViewModel documentViewModel)
        {
            Documents.Add(documentViewModel);
            documentViewModel.Task = this;
        }
    }
}
